/*
 * defines.h
 *
 * Copyright (c) 2020-2023, DarkMatterCore <pabloacurielz@gmail.com>.
 * Copyright (c) 2024, MIG Switch.
 *
 * This file is part of MigDumpTool (https://migswitch.com).
 *
 * MigDumpTool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MigDumpTool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#ifndef __DEFINES_H__
#define __DEFINES_H__

/* Broadly useful language defines. */

#define MEMBER_SIZE(type, member)       sizeof(((type*)NULL)->member)

#define MAX_ELEMENTS(x)                 ((sizeof((x))) / (sizeof((x)[0])))

#define ALIGN_UP(x, y)                  (((x) + ((y) - 1)) & ~((y) - 1))
#define ALIGN_DOWN(x, y)                ((x) & ~((y) - 1))
#define IS_ALIGNED(x, y)                (((x) & ((y) - 1)) == 0)

#define IS_POWER_OF_TWO(x)              ((x) > 0 && ((x) & ((x) - 1)) == 0)

#define DIVIDE_UP(x, y)                 (((x) + ((y) - 1)) / (y))

#define CONCATENATE_IMPL(s1, s2)        s1##s2
#define CONCATENATE(s1, s2)             CONCATENATE_IMPL(s1, s2)

#define ANONYMOUS_VARIABLE(pref)        CONCATENATE(pref, __LINE__)

#define NON_COPYABLE(cls) \
    cls(const cls&) = delete; \
    cls& operator=(const cls&) = delete

#define NON_MOVEABLE(cls) \
    cls(cls&&) = delete; \
    cls& operator=(cls&&) = delete

#define ALWAYS_INLINE                   inline __attribute__((always_inline))
#define ALWAYS_INLINE_LAMBDA            __attribute__((always_inline))

#define CLEANUP(func)                   __attribute__((__cleanup__(func)))

#define MDT_ASSERT(name, size)         static_assert(sizeof(name) == (size), "Bad size for " #name "! Expected " #size ".")

/* Global constants used throughout the application. */

#define SIXTY_FPS_DELAY                 (u64)16666667

#define FS_SYSMODULE_TID                (u64)0x0100000000000000
#define BOOT_SYSMODULE_TID              (u64)0x0100000000000005
#define SPL_SYSMODULE_TID               (u64)0x0100000000000028
#define SYSTEM_UPDATE_TID               (u64)0x0100000000000816

#define FAT32_FILESIZE_LIMIT            (u64)0xFFFFFFFF                                                 /* 4 GiB - 1 (4294967295 bytes). */

#define UTF8_BOM                        "\xEF\xBB\xBF"
#define CRLF                            "\r\n"

#define DEVOPTAB_SDMC_DEVICE            "sdmc:"

#define CONFIG_FILE_NAME                APP_TITLE "_config.json"
#define DEFAULT_CONFIG_PATH             "romfs:/default_config.json"

#define LOG_FILE_NAME                   APP_TITLE ".log"
#define LOG_BUF_SIZE                    0x400000                                                        /* 4 MiB. */
#define LOG_FORCE_FLUSH                 0                                                               /* Forces a log buffer flush each time the logfile is written to. */

#endif  /* __DEFINES_H__ */
